import pytest


@pytest.fixture()
def long_path():
    return "/this/is/a/very/long/path/to/a/file.txt"


def test_truncate_path(long_path: str):
    from alleima_tools.utils.strings import truncate_path

    assert len(truncate_path(long_path, 20)) == 20

    print(truncate_path(long_path, 20))

    # assert truncate_path(long_path, 20) == "/this/.../file.txt"
    # assert truncate_path(long_path, 30) == "/this/is/a/.../file.txt"
    # assert truncate_path(long_path, 40) == "/this/is/a/very/.../file.txt"
    # assert truncate_path(long_path, 50) == "/this/is/a/very/long/path/to/a/file.txt"



#!/usr/bin/env python
"""Tests for `alleima_tools` package."""

from decimal import Decimal

import pytest

from alleima_tools.alleima_tools import (
    distance,
    make_decimal,
    number_isclose,
)


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        (10.5, Decimal("10.5")),
        (Decimal("10.5"), Decimal("10.5")),
        ("10.5", Decimal("10.5")),
    ],
)
def test_make_decimal(value, expected):
    """Test the make_decimal function."""
    assert make_decimal(value) == expected


@pytest.mark.parametrize(
    ("value", "places", "expected"),
    [
        (10.555, 2, Decimal("10.56")),
        (10.555, 1, Decimal("10.6")),
        (10.555, 0, Decimal("11")),
        (10.555, None, Decimal("10.555")),
        (Decimal("10.555"), 2, Decimal("10.56")),
        (Decimal("10.555"), 1, Decimal("10.6")),
        (Decimal("10.555"), 0, Decimal("11")),
        (Decimal("10.555"), None, Decimal("10.555")),
        ("10.555", 2, Decimal("10.56")),
        ("10.555", 1, Decimal("10.6")),
        ("10.555", 0, Decimal("11")),
        ("10.555", None, Decimal("10.555")),
        (10.499, 2, Decimal("10.50")),
        (10.499, 1, Decimal("10.5")),
        (10.499, 0, Decimal("10")),
        (10.499, None, Decimal("10.499")),
    ],
)
def test_round_make_decimal(value, places, expected):
    """Test the make_decimal function with rounding."""
    assert make_decimal(value, places) == expected


@pytest.mark.parametrize(
    ("first", "second", "tolerance", "expected"),
    [
        (10.5, Decimal("10.5"), None, True),
        (Decimal("10.5"), Decimal("10.5"), None, True),
        ("10.5", Decimal("10.5"), None, True),
        ("10.5", 10.5001, 1e-3, True),
        (10.5, 10.5001, None, False),
    ],
)
def test_is_number_close(first, second, tolerance, expected):
    """Test the is_number_close function."""
    assert number_isclose(first, second, tolerance=tolerance) == expected


@pytest.mark.parametrize(
    ("first", "second", "expected"),
    [
        ((0, 0), (3, 4), 5.0),
        ((0, 0, 0), (3, 4, 0), 5.0),
        ((0, 0, 0), (2, 3, 4), 5.385164807134504),
        ((Decimal("0"), Decimal("0")), (Decimal("3"), Decimal("4")), Decimal("5.0")),
    ],
)
def test_distance(first, second, expected):
    """Test the distance function."""
    assert distance(first, second) == expected


def test_distance_mismatched_dimensions():
    """Test the distance function with mismatched dimensions."""
    with pytest.raises(ValueError, match="dimensions"):
        distance((0, 0), (3, 4, 0))



import math
from contextlib import nullcontext
from decimal import Decimal

import pytest

from alleima_tools.alleima_tools import number_isclose
from alleima_tools.wrappers import Q_
from alleima_tools.wrappers.pint import pint_acos, pint_asin, pint_cos

VALUE_ERROR_MSG_PATTERN = r"strict=True"


@pytest.mark.parametrize(
    ("value", "func", "expected"),
    [
        (Q_("0 degree"), pint_cos, nullcontext(1)),
        (
            math.pi,
            pint_cos,
            pytest.raises(ValueError, match=VALUE_ERROR_MSG_PATTERN),
        ),
        (Q_(math.pi / 2, "radian"), pint_cos, nullcontext(0)),
        (Decimal(".5"), pint_acos, nullcontext(Q_(math.pi / 3, "radian"))),
        ("1", pint_asin, pytest.raises(TypeError, match="not str")),
    ],
)
def test_pint_cos(value, func, expected):
    """Test the pint_cos function."""
    with expected as e:
        assert number_isclose(func(value), e)


@pytest.mark.skip(reason="Useless test of a math function.")
def test_math_sin_str():
    from math import sin

    with pytest.raises(TypeError, match="not str") as e:
        assert sin("1") == e



from collections.abc import Callable

import pytest

from alleima_tools.querylist import query_list


@pytest.mark.parametrize(
    ("full_list", "expression", "expected"),
    [
        ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], lambda x: x % 2 == 0, [2, 4, 6, 8, 10]),
        ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], lambda x: x > 10, []),  # noqa: PLR2004
        ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], lambda x: x < 0, []),
        (
            [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
            lambda x: x > 0,
            [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        ),
    ],
)
def test_query_list(full_list: list, expression: Callable, expected: list):
    # Test case 1: Empty list
    assert query_list(full_list, expression) == expected



from decimal import Decimal

from pint.registry import Quantity, UnitRegistry


def arc_ratio(angle: Quantity, registry: UnitRegistry) -> float | Decimal:
    if not isinstance(angle, Quantity):
        msg = "The angle must be a pint.Quantity object."
        raise TypeError(msg)

    divisor_value = Decimal("360") if isinstance(angle.magnitude, Decimal) else 360.0

    simple_ratio = angle / Quantity(value=divisor_value, units=registry["degree"])
    return simple_ratio.magnitude



from pathlib import Path

from pydantic import BaseModel, RootModel
from win32com.client import Dispatch


class Shortcut(BaseModel):
    """Shortcut file information."""

    path: Path
    target: Path | None = None

    @classmethod
    def from_path(cls, path: Path | str) -> "Shortcut":
        """Create a Shortcut instance from a path.

        Args:
            path (Path): Path to the shortcut file.

        Returns:
            Shortcut instance.
        """
        if isinstance(path, str):
            path = Path(path)
        return cls(path=path, target=get_shortcut_target(path))


class Shortcuts(RootModel):
    root: list[Shortcut]

    @classmethod
    def from_folder(cls, folder_path: Path | str) -> "Shortcuts":
        """Create a Shortcuts instance from a folder path.

        Args:
            folder_path (Path): Path to the folder containing the shortcut files.

        Returns:
            Shortcuts instance.
        """
        if isinstance(folder_path, str):
            folder_path = Path(folder_path)
        return cls(
            root=[
                Shortcut.from_path(file)
                for file in folder_path.iterdir()
                if file.suffix == ".lnk"
            ]
        )

    def __getitem__(self, index: int) -> Shortcut:
        return self.root[index]

    def __iter__(self):
        return iter(self.root)


def get_shortcut_target(path: Path | str) -> Path:
    """Get the target of a Windows shortcut file.

    Args:
        path (Path): Path to the shortcut file.

    Returns:
        Target of the shortcut file.
    """
    if isinstance(path, str):
        path = Path(path)
    shell = Dispatch("WScript.Shell")
    shortcut = shell.CreateShortCut(str(path))
    return Path(shortcut.Targetpath)


if __name__ == "__main__":
    from rich.console import Console
    from rich.table import Table

    table = Table(title="Shortcut Files")

    table.add_column("Shortcut File", style="cyan")
    table.add_column("Target File", style="magenta", no_wrap=True)

    parent_folder_path = Path("G:/Engineering")
    search_folder_path = parent_folder_path / "HTML Space" / "Stores Dwgs"

    shortcuts = Shortcuts.from_folder(folder_path=search_folder_path)
    for shortcut in shortcuts:
        if shortcut.target.suffix == ".pdf" and "M04" in shortcut.target.stem:
            table.add_row(
                f"{shortcut.path.stem}",
                f"[link={shortcut.target.as_uri()}]{shortcut.target.relative_to(parent_folder_path)}[/link]",
            )

    console = Console()

    console.print(table)



from pathlib import Path


def truncate_path(path: Path | str, max_length: int):
    """Truncate a path to a maximum length.

    The path is truncated to the specified length by removing characters from the
    middle of the path.

    Examples:
        >>> truncate_path("C:/Users/username/Documents/file.txt", 20)
        'C:/Us...ents/file.txt'
        >>> truncate_path("C:/Users/username/Documents/file.txt", 30)
        'C:/Users/username/Docum.../file.txt'

    Args:
        path (Path | str): The path to be truncated.
        max_length (int): The maximum length of the truncated path.

    Returns:
        str: The truncated path.
    """
    path_str = str(path)
    if len(path_str) <= max_length:
        return path_str

    tail_length = max_length // 2
    head_length = max_length - tail_length - 3

    head = path_str[:head_length]
    tail = path_str[-tail_length:]

    return f"{head}...{tail}"


if __name__ == "__main__":
    from rich import print

    long_path = "/this/is/a/very/long/path/to/a/file.txt"
    length = 20

    print(f"{length} characters: {truncate_path(long_path, length)}")



# ruff: noqa: UP007
from pathlib import Path
from typing import Annotated, Optional

import typer
from rich import print

from alleima_tools.cli_helpers.versions import generate_build_number

app = typer.Typer()


@app.command()
def build_number():
    """Generate a build number based on the current date and time.

    The first 8 characters of the build number are the current date in the format
    'YYYYMMDD'. The last 3 characters ara counter of 2 minute intervals since midnight.

    Examples:

        >>> generate_build_number()

        '20220131120'
    """
    print(generate_build_number())


@app.command()
def shortcuts(
    folder_path: Annotated[Path, typer.Argument(help="Path to the folder to search.")],
    target_extension: Annotated[
        Optional[str],
        typer.Option(help="Target file extension without the dot."),
    ] = "pdf",
    name_filter: Annotated[
        Optional[str], typer.Option(help="Filter the target file name.")
    ] = None,
):
    """Print a linked table of the targets of Windows shortcuts in a folder."""
    from rich.console import Console
    from rich.table import Table

    from alleima_tools.utils.win32 import Shortcuts

    table = Table(title="Shortcut Files")

    table.add_column("Shortcut File", style="cyan")
    table.add_column("Target File", style="magenta", no_wrap=True)

    shortcuts = Shortcuts.from_folder(folder_path=folder_path)
    for shortcut in shortcuts:
        if shortcut.target.suffix == f".{target_extension}":
            if name_filter is not None and name_filter not in shortcut.target.stem:
                continue

            table.add_row(
                f"{shortcut.path.stem}",
                f"[link={shortcut.target.as_uri()}]{shortcut.target!s:.100}[/link]",
            )

    console = Console()

    console.print(table)


if __name__ == "__main__":
    app()



from datetime import datetime

import pytz


def generate_build_number(time_zone: str = "UTC") -> str:
    """Generate a build number based on the current date and time.

    The first 8 characters of the build number are the current date in the format
    'YYYYMMDD'. The last 3 characters ara counter of 2 minute intervals since midnight.

    Examples:
        >>> generate_build_number()
        '20220131120'

    Args:
        time_zone (str, optional): The time zone to use for the build number. Defaults to 'UTC'.

    Returns:
        str: The generated build number"""  # noqa: E501
    timezone_obj = pytz.timezone(time_zone)
    now = datetime.now(timezone_obj)
    build_number = now.strftime("%Y%m%d")
    build_number += str((now.hour * 60 + now.minute) // 2).zfill(3)
    return build_number



from typing import Any

from pandas import DataFrame
from pydantic import GetCoreSchemaHandler
from pydantic_core import core_schema


class _DataFrameAnnotation:
    @classmethod
    def __get_pydantic_core_schema__(
        cls, _source_type: Any, _handler: GetCoreSchemaHandler
    ) -> core_schema.CoreSchema:
        def validate_from_dataframe(value: DataFrame) -> DataFrame:
            return value

        from_dataframe_schema = core_schema.chain_schema(
            [core_schema.no_info_plain_validator_function(validate_from_dataframe)]
        )



from enum import auto

from alleima_tools.enum import UpperStrEnum


class UpperEnum(UpperStrEnum):
    VALUE1 = auto()
    VALUE2 = auto()
    VALUE3 = auto()


def test_upper_str_enum():
    assert UpperEnum.VALUE1.value != "value1"
    assert UpperEnum.VALUE1.value == "VALUE1"
    assert UpperEnum.VALUE2.value == "VALUE2"
    assert UpperEnum.VALUE3.value == "VALUE3"



from datetime import UTC, datetime
from unittest import mock

from alleima_tools.cli_helpers import versions


@mock.patch(f"{versions.__name__}.datetime", wraps=datetime)
def test_build_number(mocker):
    mocker.now = mock.Mock(return_value=datetime(2022, 1, 31, 1, 20, tzinfo=UTC))

    build_number = versions.generate_build_number()

    assert build_number == "20220131040"



"""Main module."""

from decimal import Decimal
from typing import TypeVar


def is_number(s):
    """Tries to make a float, returns True if it succeeds, False otherwise."""
    if s is None:
        return False
    try:
        _ = float(s)
    except ValueError:
        return False
    else:
        return True


def make_decimal(value: str | float | Decimal, places: int | None = None) -> Decimal:
    """
    Convert the input value to a Decimal object.

    Parameters:
    value (str | float | Decimal): The value to be converted to a Decimal.
    places (int | None, optional): The number of decimal places to round the converted
        Decimal object to. If None, no rounding will be performed. Default is None.

    Returns:
    Decimal: The converted Decimal object.

    Raises:
    TypeError: If the input value is not a string, float, or Decimal.

    Examples:
    >>> make_decimal('10.5')
    Decimal('10.5')

    >>> make_decimal(10.5)
    Decimal('10.5')

    >>> make_decimal(Decimal('10.5'))
    Decimal('10.5')

    >>> make_decimal(10.555, 2)
    Decimal('10.56')
    """
    decimal_value = Decimal(value) if isinstance(value, str) else Decimal(str(value))

    return (
        decimal_value
        if places is None
        else decimal_value.quantize(Decimal(10) ** -places)
    )


def number_isclose(
    first: str | float | Decimal,
    second: str | float | Decimal,
    *,
    tolerance: float | None = None,
) -> bool:
    """
    Compare two numbers for approximate equality.

    Parameters:
    first (str | float | Decimal): The first number to compare.
    second (str | float | Decimal): The second number to compare.
    tolerance (float, optional): The maximum difference between the two numbers for
        them to be considered equal. Default is 1E-9.

    Returns:
    bool: True if the two numbers are approximately equal, False otherwise.

    Raises:
    TypeError: If the input values are not strings, floats, or Decimals.

    Examples:
    >>> number_isclose(10.5, 10.500000001)
    True

    >>> number_isclose(10.5, 10.50000001)
    False

    >>> number_isclose(10.5, 10.50000001, tolerance=1E-7)
    True

    >>> number_isclose(10.5, 10.5001, tolerance=1E-3)
    False
    """
    if tolerance is None:
        tolerance = 1e-9
    first_float = float(first)
    second_float = float(second)

    return abs(second_float - first_float) <= tolerance


class MismatchedDimensionsError(ValueError):
    """Exception raised when two points have different numbers of dimensions."""


T = TypeVar("T", float, int, Decimal)


def distance(point1: tuple[T, ...], point2: tuple[T, ...]) -> T:
    """Calculate euclidean distance between two points.

    Parameters:
    point1 (tuple): The first point.
    point2 (tuple): The second point.

    Returns:
    float: The distance between the two points.

    Examples:
    >>> distance((0, 0), (3, 4))
    5.0

    >>> distance((0, 0, 0), (3, 4, 0))
    5.0
    """
    if len(point1) != len(point2):
        msg = "Points must have the same number of dimensions."
        raise MismatchedDimensionsError(msg)

    pre_sqrt = sum((x - y) ** 2 for x, y in zip(point1, point2))  # noqa: B905 we've already checked the lengths

    if isinstance(pre_sqrt, Decimal):
        return pre_sqrt.sqrt()
    return pre_sqrt**0.5



from collections.abc import Callable
from typing import TypeVar

T = TypeVar("T")


class QueryList(list[T]):
    def query(self, expression):
        return QueryList([item for item in self if expression(item)])

    @property
    def first(self):
        return self[0] if self else None


def query_list(full_list: list[T], expression: Callable):
    return [item for item in full_list if expression(item)]



from math import acos, asin, atan, cos, sin, tan

from alleima_tools.wrappers import ureg

pint_cos = ureg.wraps(None, ureg.radian)(cos)
pint_sin = ureg.wraps(None, ureg.radian)(sin)
pint_tan = ureg.wraps(None, ureg.radian)(tan)

pint_acos = ureg.wraps(ureg.radian, None)(acos)
pint_asin = ureg.wraps(ureg.radian, None)(asin)
pint_atan = ureg.wraps(ureg.radian, None)(atan)
